/*
 * drivers/mmc/card/aerial/aerial_drv.h
 *
 *  Copyright (C) 2008 Nissin Systems Co.,Ltd.
 *  Copyright (C) 2008-2009 Atmark Techno, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 2008-03-05    Created by Nissin Systems Co.,Ltd.
 * 2009-06-19   Modified by Atmark Techno, Inc.
 */

#ifndef _MMC_CARD_AERIAL_DRV_H_
#define _MMC_CARD_AERIAL_DRV_H_

#include <linux/netdevice.h>
#include <linux/leds.h>

/* Driver Info */

#define AERIAL_DRVINFO				"aerial"
#define AERIAL_VERSION				"2.0.0"

/* BU1806GU SDIO IDs */

#ifndef SDIO_VENDOR_ID_ROHM
#define SDIO_VENDOR_ID_ROHM			(0x032c)
#endif

#ifndef SDIO_DEVICE_ID_ROHM_BU1806GU
#define SDIO_DEVICE_ID_ROHM_BU1806GU		(0x0120)
#endif

/* BU1806GU SDIO Function #1 Registers */

#define AERIAL_TX_PACKET_LENGTH_LSB		(0x00000)
#define AERIAL_TX_PACKET_LENGTH_MSB		(0x00001)
#define   AERIAL_PACKET_LENGTH_VALID		(1 << 15)
#define   AERIAL_PACKET_LENGTH_MASK		(0x7fff)
#define AERIAL_RX_PACKET_LENGTH_LSB		(0x00002)
#define AERIAL_RX_PACKET_LENGTH_MSB		(0x00003)
#define AERIAL_INT_MASK				(0x00004)
#define AERIAL_INT_PENDING			(0x00005)
#define AERIAL_INT_STATUS			(0x00006)
#define   AERIAL_INT_RXRCV			(1 << 0)
#define   AERIAL_INT_TXRDY			(1 << 2)
#define AERIAL_TX_BUFFER			(0x00007)
#define AERIAL_RX_BUFFER			(0x00008)

#define AERIAL_TX_CISINFO_STATUS		(0x10F6)
#define   AERIAL_CISSTAT_TXSLEEP		(1 << 0)
#define   AERIAL_CISSTAT_TXWAIT			(2 << 0)
#define   AERIAL_CISSTAT_TXACTIVE		(3 << 0)
#define   AERIAL_CISSTAT_MASK			(0x03)
#define   AERIAL_TOGGLE_SHIFT			(4)

/* Value */

#define AERIAL_CHUNK_SIZE			(512)
#define AERIAL_WRITE_SIZE			(1536)
#define AERIAL_READ_SIZE			(1536)
#define AERIAL_WRITE_WORD_SIZE			(384)
#define AERIAL_VERSION_SIZE			(128)

#define AERIAL_TX_CISPOLL_INTVAL		(0)	/* usecs */
#define AERIAL_TX_TOGGLE_TIMEOUT		(0)	/* msecs */

#define AERIAL_SCAN_TIME			(110)	/* msecs per ch */
#define AERIAL_SCAN_CHECK_INTERVAL		(130)	/* 10ms x 13ch */
#define AERIAL_SCAN_CHECK_RETRY			(AERIAL_SCAN_TIME / 10 * 2)

#define AERIAL_LINK_CHECK_INTERVAL		(1000)	/* msecs */
#define AERIAL_LINK_CHECK_RETRY			(60)

/* WID config packet structures */

#define MAX_APACKET_MSG_LEN (1660)

struct tx_args {
	struct net_device *dev;
	struct sk_buff *skb;
	struct list_head entry;
};

struct aerial_task_info {
	struct delayed_work queue;
	struct list_head tx_args_list;
	spinlock_t lock;
};

struct aerial_wid_frame {
	unsigned short wid;
	unsigned char len;
	unsigned char val[MAX_APACKET_MSG_LEN - 7];
}  __attribute__((packed));

struct aerial_wid_longframe {
	unsigned short wid;
	unsigned short len;
	unsigned char val[MAX_APACKET_MSG_LEN - 7];
}  __attribute__((packed));

struct aerial_packet_message {
	unsigned char type;
#define CTYPE_QUERY		0x51	/* Q: query */
#define CTYPE_WRITE		0x57	/* W: write */
#define CTYPE_RESPONSE		0x52	/* R: response */
#define CTYPE_INFO		0x49	/* I: info */
	unsigned char seqno;
	unsigned short len;
	unsigned char body[MAX_APACKET_MSG_LEN - 4];
} __attribute__((packed));
#define aerial_packet_message_fixed_len (4)

struct aerial_packet_header {
	unsigned short len:12;
	unsigned short type:4;
#define HTYPE_DATA_OUT		0x1
#define HTYPE_DATA_IN		0x2
#define HTYPE_CONFIG_RES	0x3
#define HTYPE_CONFIG_REQ	0x4
} __attribute__((packed));

struct aerial_packet {
	/* header part */
	union {
		unsigned short header;
		struct aerial_packet_header h;
	} __attribute__((packed));

	/* message part */
	union {
		unsigned char message[MAX_APACKET_MSG_LEN];
		struct aerial_packet_message m;
	} __attribute__((packed));
} __attribute__((packed, aligned(sizeof(long))));

#define response_ok(w) \
	((((w)->wid != WID_STATUS) || ((w)->val[0] != 1)) ? 0 : 1)

/* Aerial private structure */

struct aerial_private {
	struct net_device *netdev;
	struct sdio_func *func;

	atomic_t state;
#define AERIAL_STATE_NO_FW			0
#define AERIAL_STATE_READY			1
#define AERIAL_STATE_RUNNING			2

	atomic_t power_state;
#define AERIAL_PWR_STATE_ACTIVE			0x00
#define AERIAL_PWR_STATE_SLEEP			0x02
	atomic_t power_mngt;
#define AERIAL_PWR_MNGT_ACTIVE			0x00
#define AERIAL_PWR_MNGT_SAVE			0x01

	struct aerial_packet wid_response;
	struct aerial_packet rxbuf;
	struct aerial_task_info tx_task;
	struct task_struct *tx_thread;
	struct completion tx_thread_wait;

	struct net_device_stats net_stats;

	struct semaphore wid_req_lock;
	struct completion wid_complete;
	struct completion firmware_ready;
	int scan_check_retry;
	struct delayed_work scan_task;
	int link_check_retry;
	struct delayed_work link_task;

	int fw_load;
#define AERIAL_FWSTAND_ALONE                    0x00
#define AERIAL_FWLOAD_EXEC                      0x01
#define AERIAL_FWLOAD_AFTER                     0x02

	unsigned char *fw_image;
	unsigned long fw_size;
	int fw_type;
#define AERIAL_FWTYPE_STA			0x01
#define AERIAL_FWTYPE_TEST			0x02

#if defined(CONFIG_LEDS_TRIGGERS) && defined(ENABLE_LED_TRIGGER_EVENTS)
	char led_trig_name_active[TRIG_NAME_MAX];
	struct led_trigger *led_active;
	char led_trig_name_link[TRIG_NAME_MAX];
	struct led_trigger *led_link;
#endif
	int tx_cis_intval;
	int tx_cis_toggle_timeout;
	int tx_toggle;
#define TOGGLE_0				0x00
#define TOGGLE_1				0x01
#define TOGGLE_NONE				0x02

	struct attribute_group attr_group;
};

/* LEDs */

#if defined(CONFIG_LEDS_TRIGGERS) && defined(ENABLE_LED_TRIGGER_EVENTS)
static inline void
_aerial_led_trigger_register(struct led_trigger **tp, char *trig_name_buf,
			     char *ifn, char *name)
{
	if (!*tp) {
		*tp = kzalloc(sizeof(struct led_trigger), GFP_KERNEL);
		if (!*tp)
			return;
	}
	snprintf(trig_name_buf, TRIG_NAME_MAX, "%s:%s", ifn, name);
	(*tp)->name = trig_name_buf;
	led_trigger_register(*tp);
}
#define aerial_led_trigger_register(priv, type)				\
	_aerial_led_trigger_register(&priv->led_##type,			\
				     priv->led_trig_name_##type,	\
				     priv->netdev->dev.bus_id,		\
				     __stringify(type))			\

static inline void
_aerial_led_trigger_unregister(struct led_trigger **tp)
{
	if (*tp) {
		led_trigger_unregister(*tp);
		kfree(*tp);
		*tp = NULL;
	}
}
#define aerial_led_trigger_unregister(priv, type)			\
	_aerial_led_trigger_unregister(&priv->led_##type)

static inline void
_aerial_led_trigger_event(struct led_trigger *tp, enum led_brightness event)
{
	if (tp)
		led_trigger_event(tp, event);
}
#define aerial_led_trigger_event(priv, type, event)			\
	_aerial_led_trigger_event(priv->led_##type, event)
#else
#define aerial_led_trigger_register(priv, type)
#define aerial_led_trigger_unregister(priv, type)
#define aerial_led_trigger_event(priv, type, event)
#endif

extern int aerial_log_flag;

extern int aerial_restart(struct aerial_private *priv);
extern int aerial_send_prepare(struct sdio_func *func, int size);

extern int aerial_flow_start(struct aerial_private *priv);
extern int aerial_flow_end(struct aerial_private *priv);

/* log and dump */

extern int aerial_print(int level, const char *format, ...);
extern int aerial_dump(int level, void *buf, int len);

#define LOGLVL_1 0x1 /* EMERG, ALERT, CRIT, ERR */
#define LOGLVL_2 0x2 /* WARNING, NOTICE, INFO */
#define LOGLVL_3 0x4 /* DEBUG */

#define aer_err(args...) aerial_print(LOGLVL_1, KERN_ERR "aerial: " args)
#define aer_info(args...) aerial_print(LOGLVL_2, KERN_INFO "aerial: " args)
#define aer_debug(args...) aerial_print(LOGLVL_3, KERN_INFO "aerial: " args)

#endif /* _MMC_CARD_AERIAL_DRV_H_ */
