/*
 * drivers/mmc/card/aerial/aerial_sysfs.c
 *
 *  Copyright (C) 2008 Nissin Systems Co.,Ltd.
 *  Copyright (C) 2008-2009 Atmark Techno, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 2008-07-18    Created by Nissin Systems Co.,Ltd.
 * 2009-06-19   Modified by Atmark Techno, Inc.
 */

#include <linux/module.h>

#include "aerial_drv.h"
#include "aerial_wid.h"
#include "aerial_fw.h"

struct aerial_attribute {
	struct module_attribute mattr;
	struct aerial_private *priv;

	int flags;
#define AFLAG_NONE	(0)
#define AFLAG_SIGNED	(1)

	int (*get8)(struct aerial_private *, unsigned char *);
	int (*set8)(struct aerial_private *, unsigned char);
	int (*get16)(struct aerial_private *, unsigned short *);
	int (*set16)(struct aerial_private *, unsigned short);
	int (*get32)(struct aerial_private *, unsigned int *);
	int (*set32)(struct aerial_private *, unsigned int);

	int (*getstr)(struct aerial_private *, char *, int);
};

#define to_aerial_attr(x) container_of(x, struct aerial_attribute, mattr)

#define AERIAL_ATTR(_name, _mode, _show, _store)	\
struct aerial_attribute aer_attr_##_name = {		\
	.mattr = __ATTR(_name, _mode, _show, _store),	\
};

#define AERIAL_ATTR_COMMON8(_name, _mode, _get, _set)			\
struct aerial_attribute aer_attr_##_name = {				\
	.mattr = __ATTR(_name, _mode,					\
			aerial_sysfs_common_show,			\
			aerial_sysfs_common_store),			\
	.get8 = (_get),							\
	.set8 = (_set),							\
};
#define AERIAL_ATTR_COMMON8_SIGNED(_name, _mode, _get, _set)		\
struct aerial_attribute aer_attr_##_name = {				\
	.mattr = __ATTR(_name, _mode,					\
			aerial_sysfs_common_show,			\
			aerial_sysfs_common_store),			\
	.get8 = (_get),							\
	.set8 = (_set),							\
	.flags = AFLAG_SIGNED,						\
};
#define AERIAL_ATTR_COMMON16(_name, _mode, _get, _set)			\
struct aerial_attribute aer_attr_##_name = {				\
	.mattr = __ATTR(_name, _mode,					\
			aerial_sysfs_common_show,			\
			aerial_sysfs_common_store),			\
	.get16 = (_get),						\
	.set16 = (_set),						\
};
#define AERIAL_ATTR_COMMON32(_name, _mode, _get, _set)			\
struct aerial_attribute aer_attr_##_name = {				\
	.mattr = __ATTR(_name, _mode,					\
			aerial_sysfs_common_show,			\
			aerial_sysfs_common_store),			\
	.get32 = (_get),						\
	.set32 = (_set),						\
};
#define AERIAL_ATTR_COMMON_STR(_name, _mode, _get)			\
struct aerial_attribute aer_attr_##_name = {				\
	.mattr = __ATTR(_name, _mode,					\
			aerial_sysfs_common_string_show,		\
			NULL),						\
	.getstr = (_get),						\
};

static ssize_t
aerial_sysfs_store_firmware(struct module_attribute *mattr,
			    struct module *mod, const char *buf, size_t count)
{
	struct aerial_attribute *attr = to_aerial_attr(mattr);
	struct aerial_private *priv = attr->priv;
	struct aerial_firmware_info *info =
		(struct aerial_firmware_info *)buf;
	static struct aerial_firmware_info header;
	static loff_t pos = 0;

	if (info->id == AERIAL_FW_ID) {
		memcpy(&header, buf, sizeof(struct aerial_firmware_info));
		pos = 0;
	}

	if (pos + count > FIRMWARE_SIZE_MAX)
		return -EFBIG;

	memcpy(priv->fw_image + pos, buf, count);

	pos += count;
	priv->fw_size = pos;

	return count;
}

static ssize_t
aerial_sysfs_show_scanning(struct module_attribute *mattr,
			   struct module *mod, char *buf)
{
	struct aerial_attribute *attr = to_aerial_attr(mattr);
	struct aerial_private *priv = attr->priv;
	struct aerial_survey_res survey[2];
	unsigned char old_scantype, old_scanfilter;
	unsigned char old_ch;
	char bar[] = {"---------------------------------------"
		      "---------------------------------------"};
	char fmt[] = {"%-33s|%-5s|%-3d|%-11s|"
		      "%02x.%02x.%02x.%02x.%02x.%02x |%d\n"};
	int len = 0;
	int ret;
	int i, j;

	aerial_get_scan_type(priv, &old_scantype);
	aerial_get_scan_filter(priv, &old_scanfilter);
	aerial_get_channel(priv, &old_ch);

	aerial_set_sitesv(priv, 0x01);
	aerial_set_site_survey_scan_time(priv, AERIAL_SCAN_TIME);
	aerial_set_scan_type(priv, 0x0);
	aerial_set_scan_filter(priv, 0x0);

	aerial_set_startscan(priv, 0x01);
	for (i=0; i<AERIAL_SCAN_CHECK_RETRY; i++) {
		unsigned char val;
		aerial_get_startscan(priv, &val);
		if (!val)
			break;
		msleep(AERIAL_SCAN_CHECK_INTERVAL);
	}

	for (i=0; i<2; i++) {
		int size = sizeof(struct aerial_survey_res);
		memset(&survey[i], 0, size);
		ret = aerial_get_site_survey_results(priv, &survey[i],size);
		if (ret)
			return -EIO;
	}

	aerial_set_sitesv(priv, 0x02);

	aerial_set_scan_type(priv, old_scantype);
	aerial_set_scan_filter(priv, old_scanfilter);
	aerial_set_channel(priv, old_ch);

	len += sprintf(buf + len, "%-33s|%-5s|%-3s|%-11s|%-18s|%-4s\n",
		       "SSID", "Type", "Ch", "Security", "BSSID", "dB");
	len += sprintf(buf + len, "%s\n", bar);

	for (i=0; i<2; i++) {
		int nr_info = (survey[i].size /
			       sizeof(struct aerial_survey_info));

		for (j=0; j<nr_info; j++) {
			struct aerial_survey_info *info;
			char bsstype[][5] = {"BSS", "IBSS"};
			char *security;
			char tmp[16];

			info = &survey[i].info[j];

			switch (info->security) {
			case AERIAL_CRYPT_DISABLE:
				security = "NONE"; break;
			case AERIAL_CRYPT_UNKNOWN:
				security = "UNKNOWN"; break;
			case AERIAL_CRYPT_WEP64:
				security = "WEP64"; break;
			case AERIAL_CRYPT_WEP128:
				security = "WEP128"; break;
			case AERIAL_CRYPT_WPA_CCMP:
				security = "WPA-AES"; break;
			case AERIAL_CRYPT_WPA_TKIP:
				security = "WPA-TKIP"; break;
			case AERIAL_CRYPT_WPA_MIX:
				security = "WPA-MIX"; break;
			case AERIAL_CRYPT_WPA2_CCMP:
				security = "WPA2-AES"; break;
			case AERIAL_CRYPT_WPA2_TKIP:
				security = "WPA2-TKIP"; break;
			case AERIAL_CRYPT_WPA2_MIX:
				security = "WPA2-MIX"; break;
			case AERIAL_CRYPT_WPA_2_TKIP:
				security = "WPA/2-TKIP"; break;
			case AERIAL_CRYPT_WPA_2_CCMP:
				security = "WPA/2-AES"; break;
			case AERIAL_CRYPT_WPA_2_MIX:
				security = "WPA/2-MIX"; break;
			default:
				sprintf(tmp, "ID:%02x", info->security);
				security = tmp; break;
			}

			len += sprintf(buf + len, fmt,
				       info->ssid,
				       bsstype[info->bsstype],
				       info->channel,
				       security,
				       info->bssid[0],
				       info->bssid[1],
				       info->bssid[2],
				       info->bssid[3],
				       info->bssid[4],
				       info->bssid[5],
				       info->rxpower
				       );
		}
	}

	return len;
}

static ssize_t
aerial_sysfs_show_driver_version(struct module_attribute *mattr,
				 struct module *mod, char *buf)
{
	int len = 0;

	len += sprintf(buf + len, "%s\n", AERIAL_VERSION);

	return len;
}

static ssize_t
aerial_sysfs_common_show(struct module_attribute *mattr,
			 struct module *mod, char *buf)
{
	struct aerial_attribute *attr = to_aerial_attr(mattr);
	struct aerial_private *priv = attr->priv;
	int len = 0;
	int ret;

	if (attr->get8) {
		unsigned char val;
		ret = attr->get8(priv, &val);
		if (attr->flags & AFLAG_SIGNED)
			len += sprintf(buf + len, "%d\n", (signed char)val);
		else
			len += sprintf(buf + len, "%d\n", val);
	} else if (attr->get16) {
		unsigned short val;
		ret = attr->get16(priv, &val);
		if (attr->flags & AFLAG_SIGNED)
			len += sprintf(buf + len, "%d\n", (signed short)val);
		else
			len += sprintf(buf + len, "%d\n", val);
	} else if (attr->get32) {
		unsigned int val;
		ret = attr->get32(priv, &val);
		if (attr->flags & AFLAG_SIGNED)
			len += sprintf(buf + len, "%d\n", (signed int)val);
		else
			len += sprintf(buf + len, "%d\n", val);
	} else
		return -EOPNOTSUPP;

	if (ret)
		return ret;

	return len;
}

static ssize_t
aerial_sysfs_common_store(struct module_attribute *mattr,
			  struct module *mod, const char *buf, size_t count)
{
	struct aerial_attribute *attr = to_aerial_attr(mattr);
	struct aerial_private *priv = attr->priv;
	long val;
	int ret;

	ret = strict_strtol(buf, 0, &val);
	if (ret)
		return -EINVAL;

	if (attr->set8) {
		if (val < 0 || 255 < val)
			return -EINVAL;
		ret = attr->set8(priv, (unsigned char)val);
	} else if (attr->set16) {
		if (val < 0 || 65535 < val)
			return -EINVAL;
		ret = attr->set16(priv, (unsigned short)val);
	} else if (attr->set32)
		ret = attr->set32(priv, (unsigned int)val);
	else
		return -EOPNOTSUPP;

	if (ret)
		return ret;

	return count;
}

static ssize_t
aerial_sysfs_common_string_show(struct module_attribute *mattr,
				struct module *mod, char *buf)
{
	struct aerial_attribute *attr = to_aerial_attr(mattr);
	struct aerial_private *priv = attr->priv;
	char val[2048];
	int len = 0;
	int ret;

	memset(val, 0, 2048);
	if (attr->getstr)
		ret = attr->getstr(priv, val, 2048);
	else
		return -EOPNOTSUPP;

	if (ret)
		return ret;

	len += sprintf(buf + len, "%s\n", val);

	return len;
}

/* aerial_attributes */

static AERIAL_ATTR(firmware, 0200,
		   NULL,
		   aerial_sysfs_store_firmware);
static AERIAL_ATTR(scanning, 0444,
		   aerial_sysfs_show_scanning,
		   NULL);

static AERIAL_ATTR_COMMON8(bss_type, 0644,
			   aerial_get_bsstype,
			   aerial_set_bsstype);
static AERIAL_ATTR_COMMON16(beacon_interval, 0644,
			    aerial_get_beaconint,
			    aerial_set_beaconint);
static AERIAL_ATTR_COMMON8(listen_interval, 0644,
			   aerial_get_lisnintvl,
			   aerial_set_lisnintvl);
static AERIAL_ATTR_COMMON8(antenna_select, 0644,
			   aerial_get_antenna,
			   aerial_set_antenna);
static AERIAL_ATTR_COMMON8_SIGNED(rssi, 0444,
				  aerial_get_rssi,
				  NULL);
static AERIAL_ATTR_COMMON8(current_mac_status, 0444,
			   aerial_get_current_mac_status,
			   NULL);
static AERIAL_ATTR_COMMON_STR(firmware_version, 0444,
			      aerial_get_firmware_version);
static AERIAL_ATTR_COMMON_STR(serial_number, 0444,
			      aerial_get_serial_number);
static AERIAL_ATTR(driver_version, 0444,
		   aerial_sysfs_show_driver_version,
		   NULL);

static struct aerial_attribute *aerial_attrs[] = {
	&aer_attr_firmware,
	&aer_attr_scanning,

	&aer_attr_bss_type,
	&aer_attr_beacon_interval,
	&aer_attr_listen_interval,
	&aer_attr_antenna_select,
	&aer_attr_rssi,
	&aer_attr_current_mac_status,
	&aer_attr_firmware_version,
	&aer_attr_serial_number,
	&aer_attr_driver_version,
};

int aerial_sysfs_init(struct aerial_private *priv)
{
	int ret;
	int i;

	priv->attr_group.name = priv->netdev->name;
	priv->attr_group.attrs = kzalloc(sizeof(struct attribute *) *
					 (ARRAY_SIZE(aerial_attrs) + 1),
					 GFP_KERNEL);
	if (!priv->attr_group.attrs)
		return -ENOMEM;

	for (i=0; i<ARRAY_SIZE(aerial_attrs); i++) {
		aerial_attrs[i]->priv = priv;
		priv->attr_group.attrs[i] = &aerial_attrs[i]->mattr.attr;
	}

	if (get_device(&priv->netdev->dev)) {
		ret = sysfs_create_group(&THIS_MODULE->mkobj.kobj,
					 &priv->attr_group);
		if (ret) {
			kfree(priv->attr_group.attrs);
			return ret;
		}
		put_device(&priv->netdev->dev);
	}

	return 0;
}

void aerial_sysfs_remove(struct aerial_private *priv)
{
	sysfs_remove_group(&THIS_MODULE->mkobj.kobj, &priv->attr_group);

	kfree(priv->attr_group.attrs);

	return;
}
