/*
 * drivers/mmc/card/aerial/aerial_wid.h
 *
 *  Copyright (C) 2008 Nissin Systems Co.,Ltd.
 *  Copyright (C) 2008-2009 Atmark Techno, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 2008-06-17    Created by Nissin Systems Co.,Ltd.
 * 2009-06-19   Modified by Atmark Techno, Inc.
 */

#ifndef _MMC_CARD_AERIAL_WID_H_
#define _MMC_CARD_AERIAL_WID_H_

#include "aerial_drv.h"

/* WIDs & parameters */

enum {
	WID_BSS_TYPE				= 0x0000,
#define AERIAL_BSS_INFRA		0
#define AERIAL_BSS_ADHOC		1
#define AERIAL_BSS_AP			2
#define AERIAL_BSS_WLANOFF		0x80
	WID_CURRENT_TX_RATE			= 0x0001,
#define AERIAL_TXRATE_AUTO		0
#define AERIAL_TXRATE_MAX		54
	WID_CURRENT_CHANNEL			= 0x0002,
#define AERIAL_CHANNEL_MIN		1
#define AERIAL_CHANNEL_MAX		13
	WID_STATUS				= 0x0005,
	WID_SCAN_TYPE				= 0x0007,
#define AERIAL_SCAN_TYPE_PASSIVE	0
#define AERIAL_SCAN_TYPE_ACTIVE		1
	WID_POWER_MANAGEMENT			= 0x000B,
#define AERIAL_PMMODE_ACT		0
#define AERIAL_PMMODE_SAVE		1
	WID_802_11I_MODE			= 0x000C,
	WID_SITE_SURVEY				= 0x000E,
#define AERIAL_SITESV_DISABLE		0
#define AERIAL_SITESV_ENABLE		1
	WID_LISTEN_INTERVAL			= 0x000F,
#define AERIAL_LISTENINTVL_MIN		1
#define AERIAL_LISTENINTVL_MAX		255
	WID_START_SCAN_REQ			= 0x001E,
#define AERIAL_SCAN_OFF			0
#define AERIAL_SCAN_ON			1
	WID_RSSI				= 0x001F,
	WID_JOIN_REQ				= 0x0020,
	WID_ANTENNA_SELECTION			= 0x0021,
	WID_POWER_SAVE				= 0x0028,
#define AERIAL_POWER_ACT		0
#define AERIAL_POWER_STBY		1
#define AERIAL_POWER_SLEEP		2
	WID_CURRENT_MAC_STATUS			= 0x0031,
	WID_SCAN_FILTER				= 0x0036,
	WID_DEVICE_READY			= 0x003D,
	WID_802_11I_KEY_AUTO_GENERATION		= 0x0051,
	WID_DOWNLOAD_REQUEST			= 0x00F0,/* For Firmware */
	WID_START				= 0x00F1,/* For Firmware */
	WID_RESET				= 0x00F2,/* For Firmware */
	WID_BEACON_INTERVAL			= 0x1006,
#define AERIAL_BEACONINTVL_MIN		1
#define AERIAL_BEACONINTVL_MAX		65535
	WID_SITE_SURVEY_SCAN_TIME		= 0x100E,
	WID_ENABLE_CHANNEL			= 0x2024,
	WID_SSID				= 0x3000,
#define AERIAL_SSID_MAX			32
	WID_FIRMWARE_VERSION			= 0x3001,
	WID_BSSID				= 0x3003,
	WID_WEP_KEY_VALUE0			= 0x3004,
#define AERIAL_WEPKEY_MIN		10
#define AERIAL_WEPKEY_MAX		26
	WID_802_11I_PSK				= 0x3008,
#define AERIAL_PSKPASS_MIN		8
#define AERIAL_PSKPASS_MAX		63
	WID_MAC_ADDR				= 0x300C,
	WID_SITE_SURVEY_RESULTS			= 0x3012,
	WID_SERIAL_NUMBER			= 0x3018,
	WID_802_11I_PMK				= 0x3030,
	WID_DOWNLOAD_DATA			= 0x30F0,/* For Firmware */
};

#define AERIAL_KEYINDEX_MAX		1

/* site survey structures */

struct aerial_survey_info {
	unsigned char ssid[33];
	unsigned char bsstype;
	unsigned char channel;
	unsigned char security;
#define AERIAL_CRYPT_ENABLE	(1<<0)
#define AERIAL_CRYPT_WEP	(1<<1)
#define AERIAL_CRYPT_WEPSZ	(1<<2)
#define AERIAL_CRYPT_WPA	(1<<3)
#define AERIAL_CRYPT_WPA2	(1<<4)
#define AERIAL_CRYPT_CCMP	(1<<5)
#define AERIAL_CRYPT_TKIP	(1<<6)
#define AERIAL_CRYPT_RESERVED	(1<<7)

#define AERIAL_CRYPT_WEP_MASK	(AERIAL_CRYPT_WEP | AERIAL_CRYPT_WEPSZ)
#define AERIAL_CRYPT_WPA_MASK	(AERIAL_CRYPT_WPA | AERIAL_CRYPT_WPA2 | \
				 AERIAL_CRYPT_CCMP | AERIAL_CRYPT_TKIP)

#define AERIAL_CRYPT_DISABLE	(0)
#define AERIAL_CRYPT_UNKNOWN	(AERIAL_CRYPT_ENABLE)
#define AERIAL_CRYPT_WEP64	(AERIAL_CRYPT_ENABLE | AERIAL_CRYPT_WEP)
#define AERIAL_CRYPT_WEP128	(AERIAL_CRYPT_ENABLE | AERIAL_CRYPT_WEP | \
				 AERIAL_CRYPT_WEPSZ)
#define AERIAL_CRYPT_WPA_CCMP	(AERIAL_CRYPT_ENABLE | AERIAL_CRYPT_WPA | \
				 AERIAL_CRYPT_CCMP)
#define AERIAL_CRYPT_WPA_TKIP	(AERIAL_CRYPT_ENABLE | AERIAL_CRYPT_WPA | \
				 AERIAL_CRYPT_TKIP)
#define AERIAL_CRYPT_WPA_MIX	(AERIAL_CRYPT_ENABLE | AERIAL_CRYPT_WPA | \
				 AERIAL_CRYPT_CCMP | AERIAL_CRYPT_TKIP)
#define AERIAL_CRYPT_WPA2_CCMP	(AERIAL_CRYPT_ENABLE | AERIAL_CRYPT_WPA2 | \
				 AERIAL_CRYPT_CCMP)
#define AERIAL_CRYPT_WPA2_TKIP	(AERIAL_CRYPT_ENABLE | AERIAL_CRYPT_WPA2 | \
				 AERIAL_CRYPT_TKIP)
#define AERIAL_CRYPT_WPA2_MIX	(AERIAL_CRYPT_ENABLE | AERIAL_CRYPT_WPA2 | \
				 AERIAL_CRYPT_CCMP | AERIAL_CRYPT_TKIP)
#define AERIAL_CRYPT_WPA_2_CCMP	(AERIAL_CRYPT_ENABLE | \
				 AERIAL_CRYPT_WPA | AERIAL_CRYPT_WPA2 | \
				 AERIAL_CRYPT_CCMP)
#define AERIAL_CRYPT_WPA_2_TKIP	(AERIAL_CRYPT_ENABLE | \
				 AERIAL_CRYPT_WPA | AERIAL_CRYPT_WPA2 | \
				 AERIAL_CRYPT_TKIP)
#define AERIAL_CRYPT_WPA_2_MIX	(AERIAL_CRYPT_ENABLE | \
				 AERIAL_CRYPT_WPA | AERIAL_CRYPT_WPA2 | \
				 AERIAL_CRYPT_CCMP | AERIAL_CRYPT_TKIP)

	unsigned char bssid[6];
	signed char rxpower;
	unsigned char reserved;
} __attribute__((packed));

struct aerial_survey_res {
	unsigned char size;
	unsigned char index;
	struct aerial_survey_info info[5];
} __attribute__((packed));

/* externs */

extern int aerial_set_bsstype(struct aerial_private *priv,
			      unsigned char type);
extern int aerial_get_bsstype(struct aerial_private *priv,
			      unsigned char *type);
extern int aerial_set_ssid(struct aerial_private *priv,
			   char *ssid, int size);
extern int aerial_get_ssid(struct aerial_private *priv,
			   char *ssid, int size);
extern int aerial_get_bssid(struct aerial_private *priv,
			    unsigned char *bssid, int size);
extern int aerial_set_channel(struct aerial_private *priv,
			      unsigned char ch);
extern int aerial_get_channel(struct aerial_private *priv,
			      unsigned char *ch);
extern int aerial_set_macaddr(struct aerial_private *priv,
			      unsigned char *mac, int size);
extern int aerial_get_macaddr(struct aerial_private *priv,
			      unsigned char *mac, int size);
extern int aerial_set_rate(struct aerial_private *priv,
			   unsigned char rate);
extern int aerial_get_rate(struct aerial_private *priv,
			   unsigned char *rate);
extern int aerial_set_cryptmode(struct aerial_private *priv,
				unsigned char mode);
extern int aerial_get_cryptmode(struct aerial_private *priv,
				unsigned char *mode);
extern int aerial_set_wepkey(struct aerial_private *priv,
			     int index, char *key, int len);
extern int aerial_get_wepkey(struct aerial_private *priv,
			     int index, char *key, int len);
extern int aerial_set_powerman(struct aerial_private *priv,
			       unsigned char mode);
extern int aerial_get_powerman(struct aerial_private *priv,
			       unsigned char *mode);
extern int aerial_set_scan_type(struct aerial_private *priv,
				unsigned char type);
extern int aerial_get_scan_type(struct aerial_private *priv,
				unsigned char *type);
extern int aerial_set_sitesv(struct aerial_private *priv,
			     unsigned char mode);
extern int aerial_get_sitesv(struct aerial_private *priv,
			     unsigned char *mode);
extern int aerial_set_psctl(struct aerial_private *priv,
			    unsigned char mode);
extern int aerial_set_startscan(struct aerial_private *priv,
				unsigned char mode);
extern int aerial_get_startscan(struct aerial_private *priv,
				unsigned char *mode);
extern int aerial_set_beaconint(struct aerial_private *priv,
				unsigned short val);
extern int aerial_get_beaconint(struct aerial_private *priv,
				unsigned short *val);
extern int aerial_set_lisnintvl(struct aerial_private *priv,
				unsigned char val);
extern int aerial_get_lisnintvl(struct aerial_private *priv,
				unsigned char *val);
extern int aerial_set_passps(struct aerial_private *priv,
			     char *passps, int size);
extern int aerial_set_antenna(struct aerial_private *priv,
			      unsigned char mode);
extern int aerial_get_antenna(struct aerial_private *priv,
			      unsigned char *mode);
extern int aerial_set_join(struct aerial_private *priv, unsigned char index);
extern int aerial_get_firmware_version(struct aerial_private *priv,
				       char *buf, int size);
extern int aerial_get_site_survey_results(struct aerial_private *priv,
					  struct aerial_survey_res *buf,
					  int size);
extern int aerial_get_rssi(struct aerial_private *priv,
			   unsigned char *rssi);
extern int aerial_get_current_mac_status(struct aerial_private *priv,
					 unsigned char *val);
extern int aerial_get_serial_number(struct aerial_private *priv,
				    char *buf, int size);
extern int aerial_set_scan_filter(struct aerial_private *priv,
				  unsigned char val);
extern int aerial_get_scan_filter(struct aerial_private *priv,
				  unsigned char *val);
extern int aerial_set_site_survey_scan_time(struct aerial_private *priv,
					    unsigned short val);
extern int aerial_get_site_survey_scan_time(struct aerial_private *priv,
					    unsigned short *val);
extern int aerial_set_pmk(struct aerial_private *priv, char *key, int size);
extern int aerial_set_key_generation(struct aerial_private *priv,
				     unsigned char mode);
extern int aerial_get_key_generation(struct aerial_private *priv,
				     unsigned char *mode);

extern void set_wid_query_request(struct aerial_packet *apacket,
				  unsigned short wid);
extern void set_wid_write_request(struct aerial_packet *apacket,
				  unsigned short wid,
				  void *val,
				  unsigned char size);
extern int aerial_wid_request(struct sdio_func *func,
			      struct aerial_packet *req,
			      struct aerial_packet *res,
			      unsigned int timeout_ms);

#define to_wid_frame(p) ((struct aerial_wid_frame *)(p)->m.body)
#define to_wid_longframe(p) ((struct aerial_wid_longframe *)(p)->m.body)

#endif /* _MMC_CARD_AERIAL_WID_H_ */
